package org.apache.bcel.generic;

import org.apache.bcel.util.ByteSequence;
import java.io.IOException;
import java.io.DataOutputStream;

public class LOOKUPSWITCH extends Select
{
    LOOKUPSWITCH() {
    }
    
    public LOOKUPSWITCH(final int[] match, final InstructionHandle[] targets, final InstructionHandle defaultTarget) {
        super((short)171, match, targets, defaultTarget);
        final short length = (short)(9 + this.getMatchLength() * 8);
        super.setLength(length);
        this.setFixedLength(length);
    }
    
    @Override
    public void accept(final Visitor v) {
        v.visitVariableLengthInstruction(this);
        v.visitStackConsumer(this);
        v.visitBranchInstruction(this);
        v.visitSelect(this);
        v.visitLOOKUPSWITCH(this);
    }
    
    @Override
    public void dump(final DataOutputStream out) throws IOException {
        super.dump(out);
        final int matchLength = this.getMatchLength();
        out.writeInt(matchLength);
        for (int i = 0; i < matchLength; ++i) {
            out.writeInt(super.getMatch(i));
            out.writeInt(this.setIndices(i, this.getTargetOffset(super.getTarget(i))));
        }
    }
    
    @Override
    protected void initFromFile(final ByteSequence bytes, final boolean wide) throws IOException {
        super.initFromFile(bytes, wide);
        final int matchLength = bytes.readInt();
        this.setMatchLength(matchLength);
        final short fixedLength = (short)(9 + matchLength * 8);
        this.setFixedLength(fixedLength);
        final short length = (short)(matchLength + super.getPadding());
        super.setLength(length);
        super.setMatches(new int[matchLength]);
        super.setIndices(new int[matchLength]);
        super.setTargets(new InstructionHandle[matchLength]);
        for (int i = 0; i < matchLength; ++i) {
            super.setMatch(i, bytes.readInt());
            super.setIndices(i, bytes.readInt());
        }
    }
}
